<?php
// Implement a simple Captcha form protection.
// This requires two MySQL tables, optionally named capcha_word and capcha_session.  
// capcha_words has one column, sWord, which contains a 5-to-7-letter word.  
//    This is our source for random words.
// capcha_sess contains three columns:
//    sessID which is a 32 character unique key for the Capcha request.
//    requestTime which is a timestamp which lets us clean up old requests.
//    sWord which it this request's target word.
// Edit this file to set the database connection parameters.

class Captcha {
  // Edit this to fit your suituation.
  var $dbHost       = "localhost";
  var $dbUser       = "captcha";
  var $dbPass       = "whatever";
  var $dbDbName     = "capcha";
  var $dbWordTable  = "captcha_word";
  var $dbSessTable  = "captcha_session";
  
  // full path to TrueType font.
  var $fontFile     = "/usr/share/fonts/FreeSerifBoldItalic.ttf";  
  
  // URL of the word-drawing script.  Can be fully qualified, absolute, or relative, 
  // but if it is not in the same domain as the form, browsers may not call it.
  var $drawScript   = "/scripts/CaptchaDraw.php";
  
  var $sessionLife  = 1200;  // How long we"ll wait for a response, in seconds.
  var $pointSize    = 24;    // Size of the text.
                      
  var $error;       // Examine this for an error message.

  var $_db;
  
  // This returns HTML that can be included in your form. 
  // It assumes that the session ID is in $_REQUEST['captcha_sessID'], but you can 
  // override that by passing in a session ID.
  // You can also override the names of the word and sessID inputs, but then you
  // must make the same change in you word-drawing script and you must pass the value of
  // the session ID to this and other functions.
  function FormHTML( $sessID = false ) {
    if( ! $sessID ) {
      $sessID = $_REQUEST['captcha_sessID'];
    }
    if( ! $sessID ) {
      $sessID = uniqid();
    }
    
    // Build the whole image URL.
    $url = "{$this->drawScript}?captcha_sessID={$sessID}";

    // Return the captcha foprm HTML.  Change the styling here.
    return 
       "<div style=\"border: 3px double #666666; float: left; padding: 3px; \">\n"
      ."Please enter this word in the  box below.<br />\n"
      ."You can type the word in upper or lower case.<br />\n"
      ."<center>\n"
      ."<img src=\"{$url}\" id=\"captcha_image\" style=\"margin: 5px 5px 5px 5px; \" />\n"
      ."</center>\n"
      ."<input type=\"hidden\" name=\"captcha_sessID\" id=\"captcha_sessID\" value=\"{$sessID}\" />\n"
      ."Enter the word here: <input type=\"text\" name=\"captcha_word\" id=\"captcha_word\" size=\"12\" maxlength=\"10\" autocomplete=\"off\" /><br />\n"
      ."<small>If you have trouble reading this, <a href=\"javascript: captcha_NewWord();\">Try a Different Word</a>\n"
      ."</div>\n";
  }
  
  // Did they enter the right word?  Returns TRUE if all is good.
  // Call this in the code that processes the form.
  function CheckWord( $word = false, $sessID = false ) {
    // Get and check the Session ID.
    if( ! $sessID ) {
      $sessID = $_REQUEST['captcha_sessID'];
    }
    if( ! $sessID ) {
      $this->error = "No session ID!";
      return false;
    }
    
    // Get the response.
    if( ! $word ) {
      $word = $_REQUEST['captcha_word'];
    }
    if( ! $word ) {
      $this->error = "No word to check!";
      return false;
    }
    
    if( ! $this->_db ) {
      $this->_Connect();
    }
    
    // Delete any expired sessions.
    $then = time() - $this->sessionLife;
    $qry = "DELETE FROM {$this->dbSessTable} WHERE requestTime < $then ";
    $qrs = mysql_query( $qry, $this->_db );

    // Get the target word.
    $qry = "SELECT sWord FROM {$this->dbSessTable} WHERE sessID = '$sessID' ";
    $qrs = mysql_query( $qry, $this->_db );
    if( ! $qrs || mysql_num_rows( $qrs ) < 1 ) {
      $this->error = "Session expired.";
      return false;
    }
    
    list( $target, ) = mysql_fetch_row( $qrs );
    
    // Clean up the input.
    $word = strtolower( str_replace( ' ', '', $word ));
    if( $word == $target ) 
      return true;
    
    // Clean up the now busted session.
    $qry = "DELETE FROM {$this->dbSessTable} WHERE sessID = '$sessID' ";
    $qrs = mysql_query( $qry, $this->_db );
    $this->error = "Wrong word.";
    return false;
  }
  
  // This will be called by the word-drawing script referenced in $this->drawScript.
  // It assumes that the session ID is in $_REQUEST['captcha_sessID'], but you can 
  // override that by passing in a session ID.
  // This function passes a JPG to the browser and exits.  If all goes well, there
  // is no return from this function.
  // This function MUST be called before ANYTHING is sent to the browser!
  // This is only called in the script named in $this->drawScript.
  function DrawWord( $sessID = false ) {
    if( ! $this->_db ) {
      $this->_Connect();
    }
    
    $word = $this->_NewWord();
  
    // Get and check the Session ID.
    if( ! $sessID ) {
      $sessID = $_REQUEST['captcha_sessID'];
    }
    if( ! $sessID ) {
      $this->error = "No session ID!";
      return false;
    }
    
    $sessID = mysql_real_escape_string( $sessID );
    
    // Save the word in the session table.
    $now = time();
    $qry = "
      REPLACE INTO {$this->dbSessTable} ( sessID, sWord, requestTime )
      VALUES ( '$sessID', '$word', $now )
    ";
    $qrs = mysql_query( $qry, $this->_db );
    
    // Upper-case the first letter to make it look nicer.
    $word{0} = strtoupper( $word{0} );
  
    // Localize some stuff for convenience.
    $fontFile   = $this->fontFile;
    $pointSize  = $this->pointSize;
    
    // Angle of text tilt.
    while( abs($angle = rand( -20, 20 )) < 4 )
      ;
    
    // Metrics of the boujnding box for the drawn text.  This does NOT
    // take descenders into account!
    $textBox = imagettfbbox( $pointSize, $angle, $fontFile, $word );

    // Metrics of a horizontal rectangle that bounds the text. This does NOT
    // take descenders into account!
    $hText = max($textBox[1], $textBox[3]) - min($textBox[5], $textBox[7]);
    $wText = max($textBox[2], $textBox[4]) - min($textBox[0], $textBox[6]);
    
    // Another text height that tries to take possible descenders into account.
    $hDesc = $hText * 1.2;
    
    // Metrics for the image that we will draw into.  Leave a little margin.
    $margin = 4;
    $wImg = $wText + 4 + $margin * 2;
    $hImg = $hDesc + $margin * 2;
    
    // Make the image object.
    $img = imagecreate( $wImg, $hImg );
    
    // Allocate colors.  The first allocated is automatically the background.
    $colorWash = imagecolorallocate ( $img , 255,255,160);
    $colorLtGreen = imagecolorallocate ( $img , 160,255,160);
    $colorRed = imagecolorallocate ( $img , 255,0,0);
    $colorBlack = imagecolorallocate ( $img , 0,0,0);
    $colorBlue = imagecolorallocate ( $img , 0,0,255);
    
    // Pick a color scheme.  Assign a color to the text, for lines from the left,
    // and for curves from the right.
    switch( rand( 0, 6 )) {
    case 0:
      $leftColor = $colorRed;
      $rightColor = $colorBlue;
      $textColor = $colorBlack;
      break;
    case 1:
      $leftColor = $colorRed;
      $rightColor = $colorBlack;
      $textColor = $colorBlue;
      break;
    case 2:
      $leftColor = $colorBlue;
      $rightColor = $colorRed;
      $textColor = $colorBlack;
      break;
    case 3:
      $leftColor = $colorBlue;
      $rightColor = $colorBlack;
      $textColor = $colorRed;
      break;
    case 4:
      $leftColor = $colorBlack;
      $rightColor = $colorBlue;
      $textColor = $colorRed;
      break;
    default:
      $leftColor = $colorBlack;
      $rightColor = $colorRed;
      $textColor = $colorBlue;
      break;
    }
    
    // Put a bunch of light green blobs on the backround.
    for( $i = 0; $i < 20; ++$i ) {
      $x = rand( -5, $wImg + 5 );
      $y = rand( -5, $hImg + 5 );
      $r = rand( 3, floor( $hImg/2) );
      imagefilledellipse( $img, $x, $y, $r, $r, $colorLtGreen );
    }
    
    // Draw the text, then fuzz it.
    $x = 4 - min($textBox[0], $textBox[6]);
    $y = $hText - max( 0, $textBox[5]-$textBox[7] ) + 2;
    imagettftext( $img, $pointSize, $angle, $x, $y, $textColor, $fontFile, $word );
    imagefilter( $img, IMG_FILTER_GAUSSIAN_BLUR );
    
    // Draw a bunch of lines converging on the left.
    $x0 = rand( -25, -50 );     // Converging point...
    $y0 = rand( -5, $hImg + 5 );
    $x1 = $wImg;        // Point on the other side to which we will draw line.
    $y1 = -2 * $hImg;   // Start from way up high.  (This is the initial Y).
    while( ($y1 += rand( 12, 30 )) < ($hImg * 3) ) {
      imageline( $img, $x0, $y0, $x1, $y1, $leftColor );
      if( rand( 0, 2 ) == 0 ) {
        imageline( $img, $x0, $y0+1, $x1, $y1+1, $leftColor );
      }
    }
    
    // Draw a bunch of ovals centered somewhere near the right-hand border.
    $x0 = rand( $wImg - 5, $wImg + 30 );    // This is the center point.
    $y0 = rand( -5, $hImg + 5 );
    $r = 0;   // Initial radius. 
    while( ($r += rand( 12, 90 )) < $wImg * 3 ) { 
      imageellipse( $img, $x0, $y0, $r, $r/2, $rightColor );
      if( rand( 0, 2 ) == 0 ) {
        imageellipse( $img, $x0, $y0, $r+1, ($r/2)+1, $rightColor );
      }
    }
    
    // Draw a frame around it all.
    imagerectangle ( $img , 0, 0, $wImg-1, $hImg-1, $colorBlack );
    imagerectangle ( $img , 1, 1, $wImg-2, $hImg-2, $colorBlack );
 
    imageinterlace( $img );
    
    // Send it and then exit.    
    header( "Content-type: image/jpeg" );
    header("Cache-Control: no-cache, must-revalidate"); 
    header("Expires: Sat, 26 Jul 1997 05:00:00 GMT"); 

    imagejpeg( $img );
    imagedestroy( $img );
    exit;
  }
  
  // Pick a random word from the database.  Returns it in lower case.
  function _NewWord() {
    if( ! $this->_db ) {
      $this->_Connect();
    }
    
    $qry = "SELECT COUNT(*) FROM {$this->dbWordTable} ";
    $qrs = mysql_query( $qry, $this->_db );
    list( $numWords, ) = mysql_fetch_row( $qrs );
    
    $place = rand( 0, $numWords - 1 );
    
    $qry = "SELECT sWord FROM {$this->dbWordTable} LIMIT 1 OFFSET $place ";
    $qrs = mysql_query( $qry, $this->_db );
    list( $word, ) = mysql_fetch_row( $qrs );
    
    return strtolower( $word );
  }
  
  // Return a database connection resource.
  function _Connect() {
    $db = @mysql_pconnect( $this->dbHost, $this->dbUser, $this->dbPass );
    if( ! $db ) {
      $this->_db = false;
      $this->error = mysql_error();
      return false;
    }
    if( ! mysql_select_db( $this->dbDbName, $db )) {
      $this->_db = false;
      $this->error = mysql_error( $db );
      return false;
    }
  
    $this->_db = $db;
    return $db;
  }
  
}
?>
